// ===============================================
// File:         read.cxx
// Description:	 
// Version 1:    
// Current:      26 Sep 1999
// Author(s):    Thomas Kern
// ===============================================

//

/*
 * Copyright (C) 1997-1998  Thomas Kern
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 */


//=====================================================================
//	INCLUDES
//=====================================================================
//
#include <stdio.h>
#include <string.h>


#ifdef _WIN32
 #include "windows.h"
#else
 #include <dirent.h>  // directory operations
 #include <values.h>
#endif


//=====================================================================
// Dateien in verschiedenen Verzeichnissen suchen
//=====================================================================
//
#define NAMELENGHT        1024
#define WORDLENGHT        256
#define MAX_DIR_ENTRIES   40

static const int maxDir = 10;
static char scannedDir[maxDir][NAMELENGHT]; // speichert die bisher genutzten Verzeichnisse
static int  dirCount = 0;

//---------------------------------------------------------------------
// ein Verzeichnis unter auslesen
void ScanDirectory(char *dir, char *pattern,
                   char  filelist[MAX_DIR_ENTRIES][NAMELENGHT],
                   int  &filecount)
{
  char  wildcard[NAMELENGHT];
  int   lenDir;
  
  // Verzeichnisnamen: Laenge und Vollstaendigkeit feststellen
  lenDir = strlen(dir);
  if(lenDir) {
    if( dir[lenDir-1] != '/' )
      dir[lenDir-1] = '/';
  }

  // den Suchstring festlegen, Initialisierung
  filecount = 0;
  sprintf(wildcard, "%s%s", dir, pattern); // dir enthaelt bereits "/"

  // -----------------------
  // nun wird's OS-abhaengig
#ifdef _WIN32

  HANDLE            searchPath = NULL;
  // LPCTSTR == const char *
  WIN32_FIND_DATA   dataStruct;
  
  searchPath = FindFirstFile( (LPCTSTR)wildcard, &dataStruct);
  // Dateinamen festhalten
  if( searchPath != INVALID_HANDLE_VALUE ) {
    strcpy(filelist[filecount], dir);
    strcat(filelist[filecount], dataStruct.cFileName);
    filecount++;
    // auf zu den naechsten
    while(    filecount < MAX_DIR_ENTRIES // Grenzen beachten
           && FindNextFile(searchPath, &dataStruct) ) { // liefert BOOL
      strcpy(filelist[filecount], dir);
      strcat(filelist[filecount], dataStruct.cFileName);
      filecount++;
    }
    FindClose(searchPath);
  }

#else // Unix
  DIR           *searchPath = NULL;
  struct dirent *dataStruct;
  int            i, test, lenf, lenp;
  char          *file;

  // Verzeichnis oeffnen
  if( (searchPath = opendir(dir)) ) { 
    while(    filecount < MAX_DIR_ENTRIES // Grenzen beachten
	   && (dataStruct = readdir(searchPath)) ) {
      file = dataStruct->d_name;
      lenf = strlen(file);      
      lenp = strlen(pattern);
      test = 0;
      for(i = 1; i < 4; i++) {
	      if( file[lenf-i] == pattern[lenp-i] )
	        test++;
      }
      if( test == 3 ) { // Dateiendung stimmt ueberein
	      strcpy(filelist[filecount], dir);
	      strcat(filelist[filecount], dataStruct->d_name);
	      filecount++;
      }
    }
    closedir(searchPath);
  }
#endif 
}

//---------------------------------------------------------------------
// Verzeichnis und Dateiname trennen
static void extractDir(char *complete, char *directory, char *file)
{
  char    tempName[NAMELENGHT];
  int     len, i, j;
  boolean haveDir = false;

  strcpy(directory, complete);
  len = strlen(directory);

  // falls kein Vezeichnis angegeben, den Rest ueberspringen
  for(i = 0; i < len; i++) {
    if( directory[i] == '/' ) {
      haveDir = true;
      break;
    }
  }

  if(!haveDir) {
    directory[0] = '\0';
    strcpy(file, complete);
  }
  else {
    // Verzeichnis und Dateinamen trennen
    i = len-1; 
    j = 0;
    while( i && directory[i] != '/' ) {
      tempName[j]  = directory[i];  // vor dem Loeschen noch speichern
      j++;
      directory[i] = '\0';
      i--;
    }

    // Dateiname muss noch in die richtige Reihenfolge gebracht werden
    tempName[j] = '\0'; // String zuende
    len = strlen(tempName);
    for(i = 0; i < len; i++) 
      file[i] = tempName[len-i-1];
    file[i] = '\0';
  
    // Verzeichnis fuer spaeteres Durchsuchen speichern
    if(!dirCount) { // erstmaliger Aufruf
      strcpy(scannedDir[dirCount], directory);
      dirCount++;
    }
    else if(dirCount < maxDir) {
      i = 0;
      j = 0;  
      while(i < dirCount) {
        j += strcmp(scannedDir[i], directory); // Verzeichnis schon bekannt ?
        i++;
      }
      if(j) {                                  // Antwort ist: nein
        strcpy(scannedDir[dirCount], directory);
        dirCount++;
      }
    }
  }
}

//---------------------------------------------------------------------
// search for a file
boolean CheckFile(char *file, char *modus)
{
  FILE    *Input = NULL;
  boolean  found = false;
  char     dirName[NAMELENGHT], fileName[NAMELENGHT];
  char     searchFile[NAMELENGHT];
  int      i;

  extractDir(file, dirName, fileName);
  i = 0;
  while( (Input == NULL) && (i < dirCount) ) {
    searchFile[0] = '\0'; // String loeschen
    strcat(searchFile, scannedDir[i]);
    strcat(searchFile, fileName);
    strcpy(file, searchFile);
    Input = fopen(file, modus);
    i++;
  }
  if( Input == NULL ) {              // nicht in Unterverzeichnissen
    strcpy(file, fileName);
    Input = fopen(file, modus);
  }
    
  // letztmalig ueberprufen 
  if(Input) {
    fclose(Input);
    found = true;
  }
  return found;
}


//=====================================================================
// Main Function: reads subdirectories for *.wrl files
//=====================================================================
//
int main(int argc, char **argv)
{  
  int         i, len1;
  const int   len3 = WORDLENGHT;
  char        myDir[len3+1];
  char        pattern[] = {"*.wrl"};
  int         DateiCount    = 0;
  char        DateiListe[MAX_DIR_ENTRIES][NAMELENGHT];
  char        StandardDir[] = {"VRML/"};


  // Einstellmoeglichkeiten per Tastatur
  printf("\n======================================\n");
  printf("This prog shows how to scan a directory with various OS\n");
  printf("======================================\n");
  fflush(stdout);

  // Verzeichnisnamen ermitteln
  if(argc > 2 && !strcmp(argv[1],"-dir") ) {
    len1 = strlen(argv[1]);
    if( argv[2][len1-1] != '/' )
      sprintf(myDir, "%s/", argv[2]);
    else
      sprintf(myDir, "%s", argv[2]);
  }
  else
    sprintf(myDir, "%s", StandardDir); // prog dir

  // Dateinamen ermitteln
  ScanDirectory(myDir, pattern, DateiListe, DateiCount);
  if(!DateiCount) { // Verzeichnis enthielt keine Dateien
    strcpy(myDir, "./");
    ScanDirectory(myDir, pattern, DateiListe, DateiCount);
  }

  // print the filenames
  printf("found %d files which matched %s-pattern:\n", DateiCount, pattern);
  for(i = 0; i < DateiCount; i++) {
    if( CheckFile(DateiListe[i], "r") ) {// file > 0 byte ??
      printf("file %d:\t %s\n", i+1, DateiListe[i]);
    }
  }

  return 0;
}

